<?php

/**
 * @file
 * Implements the views hook that announces the data.
 */

function views_schema_views_data() {
  $data = array();
  $tables = module_invoke_all('views_schema');

  foreach ($tables as $name => $table) {
    $table += array('base' => FALSE);

    if (empty($table['title'])) {
      $table['title'] = views_schema_sanitize_name($name);
    }

    $schema = schema_invoke('inspect', $name);
    $schema[$name] += $table;

    views_schema_create_definition($data, $schema[$name]);
  }

  return $data;
}


function views_schema_create_definition(&$data, $table) {
  $data[$table['name']]['table']['group'] = t('[Schema] @table', array('@table' => $table['title']));

  if ($table['base']) {
    views_schema_create_base_definition($data, $table);
  }

  foreach ($table['fields'] as $name => $field) {
    $field['name'] = $name;
    $field['title'] = views_schema_sanitize_name($name);
    $data[$table['name']][$name] = views_schema_create_column($field);
  }
}

function views_schema_create_column($field) {
  $definition = array(
    'title' => $field['title'],
    'help' => t('No help text.'),

    'field' => views_schema_create_field($field),
    'sort' => views_schema_create_sort($field),
    'filter' => views_schema_create_filter($field),
    'argument' => views_schema_create_argument($field),
  );

  return $definition;
}

function views_schema_create_base_definition(&$data, $table) {
  static $weight = 20;
  if (!empty($table['primary key'])) {
    $data[$table['name']]['table']['base'] = array(
      'field' => $table['primary key'][0],
      'title' => t('[Schema] @title', array('@title' => $table['title'])),
      'help' => t('Auto-generated from database table %name.', array('%name' => $table['name'])),
      'weight' => $weight++,
    );
  }
}

function views_schema_create_field($field) {
  $handler = array(
    'handler' => 'views_handler_field',
    'click sortable' => TRUE,
  );

  switch ($field['type']) {
    case 'serial':
    case 'int':
      $handler['handler'] = 'views_handler_field_numeric';
      break;
  }

  return $handler;
}

function views_schema_create_sort($field) {
  $handler = array(
    'handler' => 'views_handler_sort',
  );

  switch ($field['type']) {
    // No intelligent default rules.
  }

  return $handler;
}

function views_schema_create_filter($field) {
  $handler = array(
    'handler' => 'views_handler_filter',
  );

  switch ($field['type']) {
    case 'serial':
    case 'int':
      $handler['handler'] = 'views_handler_filter_numeric';
      break;
    case 'varchar':
    case 'text':
      $handler['handler'] = 'views_handler_filter_string';
      break;
  }

  return $handler;
}

function views_schema_create_argument($field) {
  $handler = array(
    'handler' => 'views_handler_argument',
  );

  switch ($field['type']) {
    case 'serial':
    case 'int':
      $handler['handler'] = 'views_handler_argument_numeric';
      break;
    case 'varchar':
    case 'text':
      $handler['handler'] = 'views_handler_argument_string';
      break;
  }

  return $handler;
}


function views_schema_sanitize_name($name) {
  return implode(' ', array_map('drupal_ucfirst', explode('_', $name)));
}
